DELIMITER //
-- -------------------------------------------------------------------------------------------
-- ====== Common =======  --
-- -------------------------------------------------------------------------------------------

-- Desc: Получает ID текущего семестра
-- Note: uses by another functions
DROP FUNCTION IF EXISTS GetCurSemesterID//
CREATE FUNCTION `GetCurSemesterID`	( ) RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT general_settings.Val 
				FROM `general_settings` 
				WHERE general_settings.ID = 1
				LIMIT 1 
			);
END //




DROP FUNCTION IF EXISTS SetCurSemesterID//
CREATE FUNCTION `SetCurSemesterID`	(	`SemesterID` INT 
									) 	RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;
	
	SELECT semesters.ID
	INTO checker
	FROM `semesters`
	WHERE semesters.ID = SemesterID
	LIMIT 1;
	IF NOT (checker > 0) THEN
		RETURN -1;
	END IF;

	UPDATE `general_settings`
	SET general_settings.Val = SemesterID
	WHERE general_settings.ID = 1;
	RETURN 0;
END //





-- Desc: Получает ID семестра
-- Note: uses by another functions
DROP FUNCTION IF EXISTS GetSemesterID//
CREATE FUNCTION `GetSemesterID`	( 	`Year` INT, 
									`Num` INT 
								) 	RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT semesters.ID 
				FROM `semesters` 
				WHERE 	semesters.Year 	= Year AND 
						semesters.Num 	= Num 
				LIMIT 1
			);
END //




-- Desc: Получает информацию о текущем семестре
DROP PROCEDURE IF EXISTS GetCurSemesterInfo//
CREATE PROCEDURE `GetCurSemesterInfo`	( )
    NO SQL
BEGIN
	SELECT 	semesters.ID 	AS 'SemesterID', 
			semesters.Num 	As 'SemesterNum', 	
			semesters.Year  As 'SemesterYear'
	FROM `general_settings`
	INNER JOIN `semesters` ON general_settings.Val = semesters.ID
	WHERE general_settings.ID = 1;
END //





DROP PROCEDURE IF EXISTS GetSemesterInfo//
CREATE PROCEDURE `GetSemesterInfo`	(	IN `SemesterID` INT
									)
    NO SQL
BEGIN
	SELECT 	semesters.Num 	As 'SemesterNum', 	
			semesters.Year  As 'SemesterYear'
	FROM `semesters`
	WHERE semesters.ID = SemesterID;
END //





DROP FUNCTION IF EXISTS SetHashKey//
CREATE FUNCTION `SetHashKey`	( 	`Value` VARCHAR(300) 
								) 	RETURNS int(11)
    NO SQL
BEGIN
	UPDATE `general_settings`
	SET general_settings.ValS = Value
	WHERE general_settings.ID = 2;
	RETURN 0;
END //




DROP FUNCTION IF EXISTS GetHashKey//
CREATE FUNCTION `GetHashKey`	( ) RETURNS varchar(300) CHARSET utf8
    NO SQL
BEGIN
	RETURN 	(	SELECT general_settings.ValS
	        	FROM `general_settings`
	        	WHERE general_settings.ID = 2
	       	);
END //




DROP FUNCTION IF EXISTS SetBitmaskByPagename//
CREATE FUNCTION `SetBitmaskByPagename`	( 	`Pagename` TEXT CHARSET utf8,
											`Mask` INT
										) 	RETURNS int(11)
    NO SQL
BEGIN
	UPDATE `page_access`
	SET page_access.Bitmask = Mask
	WHERE page_access.Pagename = Pagename
	LIMIT 1;
	RETURN 0;
END //





DROP FUNCTION IF EXISTS GetBitmaskByPagename//
CREATE FUNCTION `GetBitmaskByPagename`	(	`Pagename` TEXT CHARSET utf8
 										) 	RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT page_access.Bitmask
	        	FROM `page_access`
	        	WHERE page_access.Pagename = Pagename
	       	);
END //






-- -------------------------------------------------------------------------------------------
-- ====== base getters======
-- -------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS GetSubjects//
CREATE PROCEDURE `GetSubjects`	( )
    NO SQL
BEGIN
	SELECT 	subjects.ID 	AS 'SubjectID',
			subjects.Name 	AS 'SubjectName',
			subjects.Abbr 	AS 'SubjectAbbr'
	FROM	`subjects`
	ORDER BY subjects.Name ASC;
END //


DROP PROCEDURE IF EXISTS GetFaculties//
CREATE PROCEDURE `GetFaculties`	( )
    NO SQL
BEGIN
	SELECT 	faculties.ID 		AS 'FacultyID',
			faculties.Name   	AS 'FacultyName',
			faculties.Abbr 		AS 'FacultyAbbr'
	FROM 	`faculties`;
END //


DROP PROCEDURE IF EXISTS GetDepartments//
CREATE PROCEDURE `GetDepartments`	( )
    NO SQL
BEGIN
	SELECT 	departments.ID 		AS 'DepID',
			departments.Name 	AS 'DepName',
			faculties.ID 		AS 'FacultyID',
			faculties.Name   	AS 'FacultyName',
			faculties.Abbr 		AS 'FacultyAbbr'
	FROM `departments`
	INNER JOIN `faculties` ON faculties.ID = departments.FacultyID;
	
END //


DROP PROCEDURE IF EXISTS GetDepartmentsByFaculty//
CREATE PROCEDURE `GetDepartmentsByFaculty`	(	IN `FacultyID` INT 
											)
    NO SQL
BEGIN
	SELECT 	departments.ID 		AS 'DepID',
			departments.Name 	AS 'DepName'
	FROM `departments`
	WHERE departments.FacultyID = FacultyID;
END //



DROP PROCEDURE IF EXISTS GetSpecializations//
CREATE PROCEDURE `GetSpecializations`	(	IN `FacultyID` INT 
										)
    NO SQL
BEGIN
		SELECT 	specializations.ID 			AS 'SpecID',
				specializations.Name 		AS 'SpecName',
				specializations.Abbr 		AS 'SpecAbbr',
				specializations.FacultyID	AS 'FacultyID'
		FROM	`specializations`
		WHERE 	specializations.FacultyID = FacultyID
		ORDER BY subjects.Name ASC;
END //



DROP PROCEDURE IF EXISTS GetStudyGroups//
CREATE PROCEDURE `GetStudyGroups`	( 	IN `Grade` 		INT,
										IN `FacultyID` 	INT  
									)
    NO SQL
BEGIN
	SELECT 	study_groups.ID 			AS 'GroupID',
			study_groups.GroupNum 		AS 'GroupNum',
			specializations.ID 			AS 'SpecID',
			specializations.Name 		AS 'SpecName',
			specializations.Abbr 		AS 'SpecAbbr',
			specializations.FacultyID	AS 'FacultyID'
	FROM `study_groups`
	INNER JOIN `specializations` ON specializations.ID = study_groups.SpecializationID
	WHERE 	study_groups.Grade = Grade AND 
			specializations.FacultyID = FacultyID;
END //


DROP PROCEDURE IF EXISTS GetStudentsByStudyGroups//
CREATE PROCEDURE `GetStudentsByStudyGroups`	( 	IN `StudyGroupID` INT  
											)
    NO SQL
BEGIN
	SELECT 	students.ID 			AS 'StudentID', 
			students.LastName 		AS 'StudentLast', 
			students.FirstName 		AS 'StudentFirst',  
			students.SecondName	 	AS 'StudentSecond'
	FROM `students`
	WHERE students.StudyGroupID = StudyGroupID;

END //


DROP PROCEDURE IF EXISTS GetStudentsByFaculty//
CREATE PROCEDURE `GetStudentsByFaculty`	( 	IN `FacultyID` INT  
										)
    NO SQL
BEGIN
	SELECT 	students.ID 			AS 'StudentID', 
			students.LastName 		AS 'StudentLast', 
			students.FirstName 		AS 'StudentFirst',  
			students.SecondName	 	AS 'StudentSecond'
	FROM `students`
	INNER JOIN `study_groups` 		ON study_groups.ID = students.StudyGroupID
	INNER JOIN `specializations` 	ON study_groups.SpecializationID =  specializations.ID
	WHERE specializations.FacultyID = FacultyID;

END //



DROP PROCEDURE IF EXISTS GetTeachersByFaculty//
CREATE PROCEDURE `GetTeachersByFaculty`	( 	IN `FacultyID` INT  
										)
    NO SQL
BEGIN
	SELECT 	teachers.ID 			AS 'TeacherID', 
			teachers.LastName 		AS 'TeacherLast', 
			teachers.FirstName 		AS 'TeacherFirst', 
			teachers.SecondName		AS 'TeacherSecond', 
			degrees.Name 			AS 'DegreeName', 
			departments.ID 			AS 'DepID',
			departments.Name 		AS 'DepName'
	FROM `teachers`
	INNER JOIN `departments` ON departments.ID = teachers.DepartmentID
	INNER JOIN `degrees` ON degrees.ID = teachers.DegreeID
	WHERE departments.FacultyID = FacultyID;

END //


DROP PROCEDURE IF EXISTS GetTeachersByDepartment//
CREATE PROCEDURE `GetTeachersByDepartment`	( 	IN `FacultyID` INT  
											)
    NO SQL
BEGIN
	SELECT 	teachers.ID 			AS 'TeacherID', 
			teachers.LastName 		AS 'TeacherLast', 
			teachers.FirstName 		AS 'TeacherFirst', 
			teachers.SecondName		AS 'TeacherSecond',  
			degrees.Name 			AS 'DegreeName', 
			departments.ID 			AS 'DepartmentID',
			departments.Name 		AS 'DepartmentName'
	FROM `teachers`
	INNER JOIN `departments` ON departments.ID = teachers.DepartmentID
	INNER JOIN `degrees` ON degrees.ID = teachers.DegreeID
	WHERE departments.FacultyID = FacultyID;

END //










-- -------------------------------------------------------------------------------------------
-- ====== Работа с аккаунтами ======
-- -------------------------------------------------------------------------------------------

-- Desc: Количество аккаунтов с данным кодом активации
-- Note: uses by another functions
DROP FUNCTION IF EXISTS GetAccCountByCode//
CREATE FUNCTION `GetAccCountByCode`	( 	`Code` VARCHAR(40) CHARSET utf8 
									) 	RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT COUNT(accounts.ID) 
				FROM `accounts` 
				WHERE accounts.ActivationCode = Code 
			);
END //


-- Desc: Количество аккаунтов с данным логином
DROP FUNCTION IF EXISTS GetAccCountByMail //
CREATE FUNCTION `GetAccCountByMail`	(	`Mail` VARCHAR(50) CHARSET utf8
									) 	RETURNS int(11)
    NO SQL
BEGIN 
	RETURN 	( 	SELECT COUNT(accounts.ID) 
				FROM `accounts` 
				WHERE 	accounts.EMail = Mail
			);
END //


-- Desc: Количество аккаунтов с данным логином
DROP FUNCTION IF EXISTS GetAccCountByLogin//
CREATE FUNCTION `GetAccCountByLogin`	(	`Login` VARCHAR(50) CHARSET utf8
										) 	RETURNS int(11)
    NO SQL
BEGIN 
	RETURN 	( 	SELECT COUNT(accounts.ID) 
				FROM `accounts`
				WHERE 	accounts.Login = Login
			);
END //


DROP PROCEDURE IF EXISTS GetAccInfoByID//
CREATE PROCEDURE `GetAccInfoByID`	(	IN `UserID` VARCHAR(50) CHARSET utf8 
									)
    NO SQL
BEGIN 
	SELECT 	accounts.ID 		AS 'AccID', 
			accounts.Login 		AS 'AccLogin', 
			accounts.EMail 		AS 'AccEMail', 
			user_roles.Type 	AS 'AccType',
			user_roles.RoleName	AS 'AccRole',	 
			user_roles.Mark 	AS 'AccRoleMark',	 
			accounts.isEnabled,
			( accounts.ActivationCode IS NULL) AS 'isActivated',
			accounts.UserAgent	
	FROM `accounts`
	INNER JOIN `user_roles` ON accounts.UserRoleID = user_roles.ID
	WHERE accounts.ID = UserID;
END //


-- Desc: - UserID => FirstName, SecondName, LastName, StudyGroup (вся информация, в т.ч. и специализация (ПМ, ИТ, etc.)) (в зав. от AccountType)
-- Note: 	1) если аккаунт существует, то всегда можно получить UserID, AccountType, LastName, FirstName
--				иначе UserID = NULL (как и все остальные)
--			2) Degree у преподавателя и LastName могут быть NULL 
DROP PROCEDURE IF EXISTS GetPersonalInfoByID//
CREATE PROCEDURE `GetPersonalInfoByID`	(	IN `UserID` INT 
										)
    NO SQL
BEGIN 
	DECLARE accType INT;
	SELECT user_roles.Type
	INTO accType
	FROM `accounts`
	INNER JOIN `user_roles` ON accounts.UserRoleID = user_roles.ID 
	WHERE accounts.ID = UserID 
	LIMIT 1;
	

	IF accType = 1 THEN 
		
		SELECT 	students.ID 			AS 'StudentID', 
				students.LastName 		AS 'Last', 
				students.FirstName		AS 'First',  
				students.SecondName 	AS 'Second', 
				study_groups.ID 		AS 'GroupID', 
				study_groups.Grade 		AS 'StudentGrade', 
				study_groups.GroupNum 	AS 'GroupNum', 
				study_groups.Name 		AS 'GroupName',
				specializations.ID 		AS 'SpecID', 
				specializations.Name	AS 'SpecName', 
				specializations.Abbr 	AS 'SpecAbbr', 
				faculties.ID 			AS 'FacultyID', 
				faculties.Name 			AS 'FacultyName', 
				faculties.Abbr 			AS 'FacultyAbbr'		
		FROM `students`
		INNER JOIN `study_groups` 		ON students.StudyGroupID = study_groups.ID
		INNER JOIN `specializations` 	ON study_groups.SpecializationID = specializations.ID
		INNER JOIN `faculties` 			ON specializations.FacultyID = faculties.ID
		WHERE students.AccountID = UserID;

	ELSE
		SELECT 	teachers.ID 			AS 'TeacherID', 
				teachers.LastName 		AS 'Last', 
				teachers.FirstName 		AS 'First', 
				teachers.SecondName 	AS 'Second', 
				degrees.Name 			AS 'DegreeName', 
				departments.ID 			AS 'DepID',
				departments.Name 		AS 'DepName',
				faculties.ID 			AS 'FacultyID', 
				faculties.Name 			AS 'FacultyName', 
				faculties.Abbr 			AS 'FacultyAbbr'
		FROM `teachers`
		INNER JOIN `departments` ON departments.ID = teachers.DepartmentID
		INNER JOIN `faculties` ON departments.FacultyID = faculties.ID
		INNER JOIN `degrees` ON degrees.ID = teachers.DegreeID
		WHERE teachers.AccountID = UserID;

	END IF;

END //


-- Desc: Активация аккаунта (внесение данных изменений в поля Login, E-Mail, Password + обнуление activation_code и снятие флага isActivated). Кроме того, также установка AccountType в зависимости от связанной с ним информации.
DROP FUNCTION IF EXISTS ActivateAccount//
CREATE FUNCTION `ActivateAccount`	(	`Code` VARCHAR(40) CHARSET utf8, 
										`Login` VARCHAR(50) CHARSET utf8, 
										`Mail` VARCHAR(32) CHARSET utf8, 
										`Password` VARCHAR(255) CHARSET utf8
									) 	RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker, UserID INT;
	
	SELECT COUNT(accounts.ID) 
	INTO checker 
	FROM accounts 
	WHERE accounts.ActivationCode = Code;
	IF checker != 1 THEN
		RETURN -1;
	END IF;

	IF Password IS NULL THEN
 		return -2;
	END IF;  

	SELECT accounts.ID 
	INTO UserID 
	FROM `accounts`
	WHERE accounts.ActivationCode = Code; 

	UPDATE `accounts` 
	SET 	accounts.Login = Login , 
			accounts.Password = Password , 
			accounts.EMail = Mail , 
			accounts.ActivationCode = NULL
	WHERE accounts.ID = UserID ;
	
	RETURN UserID;

END //



-- Desc: - Изменение пароля для данной учетной записи
DROP FUNCTION IF EXISTS ChangePassword//
CREATE FUNCTION `ChangePassword`	(	`UserID` INT,  
										`Password` VARCHAR(255) CHARSET utf8
									) 	RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;

	SELECT COUNT(accounts.ID) 
	INTO checker
	FROM `accounts` 
	WHERE accounts.ID = UserID;
	IF checker != 1 THEN
		RETURN -1;
	END IF;

	UPDATE `accounts`
	SET accounts.Password = Password
	WHERE accounts.ID = UserID;

END //


-- Desc: - Изменение логина для данной учетной записи
DROP FUNCTION IF EXISTS ChangeLogin//
CREATE FUNCTION `ChangeLogin`	(	`UserID` INT,  
									`Login` VARCHAR(50) CHARSET utf8
								) 	RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;

	SELECT COUNT(accounts.ID) 
	INTO checker
	FROM `accounts` 
	WHERE accounts.ID = UserID;
	IF checker != 1 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT accounts.ID 
	INTO checker
	FROM `accounts`
	WHERE accounts.Login = Login
	LIMIT 1;
	IF checker > 0 THEN 
		RETURN -1;
	END IF;

	UPDATE `accounts` 
	SET accounts.Login = Login
	WHERE accounts.ID = UserID;
	
	RETURN 0;
END //


-- Desc: - Изменение e-mail для данной учетной записи
DROP FUNCTION IF EXISTS ChangeMail//
CREATE FUNCTION `ChangeMail`	(	`UserID` INT,  
									`EMail` VARCHAR(32) CHARSET utf8
								) 	RETURNS int(11)
    NO SQL
BEGIN
	DECLARE checker INT;
	
	SELECT COUNT(accounts.ID) 
	INTO checker
	FROM `accounts` 
	WHERE accounts.ID = UserID;
	IF checker != 1 THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT accounts.ID 
	INTO checker
	FROM accounts
	WHERE accounts.EMail = EMail
	LIMIT 1;
	IF checker > 0 THEN 
		RETURN -1;
	END IF;


	UPDATE accounts 
	SET accounts.EMail = EMail
	WHERE ID = UserID;
	
	RETURN 0;
	
END //



DROP FUNCTION IF EXISTS SignIn//
CREATE FUNCTION `SignIn`(	`Login` 	VARCHAR(255) CHARSET utf8, 
							`Password` 	VARCHAR(64) CHARSET utf8
						) 	RETURNS int(11)
    NO SQL
BEGIN 
	DECLARE curID INT;
	SELECT accounts.ID
	INTO curID
	FROM `accounts`
	WHERE 	accounts.Password = Password AND
			(	accounts.Login = Login OR 
				accounts.EMail = Login 	)
			
	LIMIT 1;

	IF curID > 0 THEN 
		RETURN curID;
	ELSE 
		RETURN -1;
	END IF;

END //













-- -------------------------------------------------------------------------------------------
-- ====== Common fillers =========
-- -------------------------------------------------------------------------------------------


-- Desc: Добавление нового студента
DROP FUNCTION IF EXISTS CreateStudent//
CREATE FUNCTION `CreateStudent`(	`Last` 			VARCHAR(30) CHARSET utf8, 
									`First` 		VARCHAR(30) CHARSET utf8, 
									`Second` 		VARCHAR(30) CHARSET utf8, 
									`Grade` 		INT, 
									`GroupN` 		INT, 
									`FacultyID` 	INT, 
									`Code` 			VARCHAR(40) CHARSET utf8 
								) 	RETURNS int(11)
    NO SQL
BEGIN 
	DECLARE curID, StudyGroupID INT;

	SELECT study_groups.ID 
	INTO StudyGroupID 
	FROM `study_groups` 
	INNER JOIN `specializations` ON study_groups.SpecializationID = specializations.ID
	WHERE 	study_groups.Grade = Grade 	AND 
			study_groups.GroupNum = GroupN AND 
			specializations.FacultyID = FacultyID 
	LIMIT 1;
	IF NOT (StudyGroupID > 0) THEN
		RETURN -1;
	END IF; 


	INSERT INTO `accounts` ( accounts.Login , accounts.Password , accounts.EMail, accounts.UserRoleID, accounts.ActivationCode )
	VALUES 				   ( NULL, NULL, NULL, 1, Code);
		
	SELECT accounts.ID 
	INTO curID
	FROM `accounts`
	WHERE accounts.ActivationCode = Code;
	
	INSERT INTO `students` (students.StudyGroupID, students.AccountID, students.LastName, students.FirstName, students.SecondName)
	VALUES 				   (StudyGroupID, curID, Last, First, Second);

	RETURN 0;

END //



DROP FUNCTION IF EXISTS CreateTeacher//
CREATE FUNCTION `CreateTeacher`(	`Last` 			VARCHAR(30) CHARSET utf8, 
									`First` 		VARCHAR(30) CHARSET utf8, 
									`Second` 		VARCHAR(30) CHARSET utf8, 
									`DegreeID` 	   	INT, 
									`DepartmentID` 	INT, 
									`Code` 			VARCHAR(40) CHARSET utf8 
								) 	RETURNS int(11)
    NO SQL
BEGIN 
	DECLARE curID, checker, RoleID INT;

	
	SELECT departments.ID 
	INTO checker
	FROM `departments`
	WHERE departments.ID = DepartmentID
	LIMIT 1;
	IF NOT (checker > 0) THEN
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT degrees.ID 
	INTO checker
	FROM `degrees`
	WHERE degrees.ID = DegreeID
	LIMIT 1;
	IF NOT (checker > 0) THEN
		RETURN -1;
	END IF;


	IF Code IS NULL THEN
		RETURN -1;
	END IF;

	INSERT INTO `accounts` ( accounts.Login , accounts.Password , accounts.EMail, accounts.UserRoleID, accounts.ActivationCode )
	VALUES 				   ( NULL, NULL, NULL, 2, Code);
	
	SELECT accounts.ID INTO curID
	FROM `accounts`
	WHERE accounts.ActivationCode = Code;
	
	INSERT INTO `teachers` (	teachers.AccountID, 
								teachers.LastName, 
								teachers.FirstName, 
								teachers.SecondName, 
								teachers.DegreeID, 
								teachers.DepartmentID)
	VALUES 				   (curID, Last, First, Second, DegreeID, DepartmentID);
	
	RETURN 0;

END //















-- -------------------------------------------------------------------------------------------
-- ======Студенты=========
-- -------------------------------------------------------------------------------------------


DROP FUNCTION IF EXISTS GetRateForDisc//
CREATE FUNCTION `GetRateForDisc`	(	`StudentID` INT,  
										`DisciplineID` INT
									) 	RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT SUM(rating_table.Rate) 
				FROM `rating_table`
				INNER JOIN `submodules` ON rating_table.SubmoduleID = submodules.ID
				INNER JOIN `modules`	ON submodules.ModuleID = modules.ID
				WHERE 	rating_table.StudentID = StudentID AND
						modules.DisciplineID = DisciplineID
			);
END //


DROP FUNCTION IF EXISTS GetMaxRateForDisc//
CREATE FUNCTION `GetMaxRateForDisc`	(	`StudentID` INT,  
										`DisciplineID` INT
									) 	RETURNS int(11)
    NO SQL
BEGIN
	RETURN 	(	SELECT SUM(submodules.MaxRate) 
				FROM `rating_table`
				INNER JOIN `submodules` ON rating_table.SubmoduleID = submodules.ID
				INNER JOIN `modules`	ON submodules.ModuleID = modules.ID
				WHERE 	rating_table.StudentID = StudentID AND
						modules.DisciplineID = DisciplineID
			);
END //




-- Desc: Все курсы, на которые подписан данный студент + преподаватель курса + тип контроля (экзамен/зачет) + баллы + максимальное количество баллов на данный момент (IN: StudentID).
DROP PROCEDURE IF EXISTS GetDisciplinesForStudent//
CREATE PROCEDURE `GetDisciplinesForStudent`(IN `StudentID` INT)
    NO SQL
BEGIN  
    DECLARE groupID, semID INT;  
    SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1;  						 
	SET semID = GetCurSemesterID();
	
	SELECT 	DISTINCT	disciplines.ID 			AS 'DisciplineID', 
						subjects.ID 			AS 'SubjectID', 
						subjects.Name 			AS 'SubjectName', 
						disciplines.ExamType,
						teachers.LastName 		AS 'TeacherLast', 
						teachers.FirstName 		AS 'TeacherFirst',
						teachers.SecondName 	AS 'TeacherSecond', 
						GetRateForDisc(StudentID, `disciplines`.`ID` ) AS 'Rate', 
						GetMaxRateForDisc(StudentID, `disciplines`.`ID` ) AS 'MaxCurrentRate' 
	FROM `disciplines`
	INNER JOIN 	`subjects` 					ON 	disciplines.SubjectID = subjects.ID 
	INNER JOIN 	`teachers` 					ON 	disciplines.AuthorID = teachers.ID
	LEFT JOIN 	`disciplines_groups` 		ON 	disciplines_groups.DisciplineID = disciplines.ID AND
												disciplines_groups.StudyGroupID = groupID
	LEFT JOIN 	`disciplines_students`		ON 	disciplines_students.DisciplineID = disciplines.ID AND
												disciplines_students.StudentID = StudentID
	WHERE 	disciplines.SemesterID = semID AND
			(	disciplines_students.Type = 'attach'	OR
				(	( disciplines_students.Type IS NULL OR disciplines_students.Type != 'detach' ) AND 
					disciplines_groups.StudyGroupID = groupID 
				)
			)
	ORDER BY disciplines.ExamType ASC, Rate DESC;    
END //



-- Decs: Полная информация по модулям/подмодулям для данного студента по данному курсу (заголовки модулей, максимальное количество баллов за подмодуль)	
DROP PROCEDURE IF EXISTS GetMapForStudent//
CREATE PROCEDURE `GetMapForStudent`	(	IN `StudentID` INT, 
										IN `SubjectID` INT
									)
BEGIN  
    DECLARE groupID, semID INT;  
    SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1;  						 
	SET semID = GetCurSemesterID();

	SELECT 	modules.ID 				AS 'ModuleID', 
			modules.Name 			AS 'ModuleName', 
			submodules.ID 			AS 'SubmoduleID', 
			submodules.Name 		AS 'SubModuleName', 
			submodules.Description 	AS 'SubmoduleDescription', 
			submodules.MaxRate,
			submodules.Type 		AS 'SubmoduleControl',
			rating_table.Rate, 
			rating_table.Date
	FROM `submodules`
	INNER JOIN `modules` ON submodules.ModuleID = modules.ID
	LEFT JOIN `rating_table` ON submodules.ID = rating_table.SubModuleID AND rating_table.StudentID = StudentID 
	WHERE modules.DisciplineID IN 
		(		SELECT DISTINCT	disciplines.ID 
				FROM `disciplines_teachers`
				INNER JOIN `disciplines` 				ON 	disciplines_teachers.DisciplineID = disciplines.ID
				LEFT JOIN 	`disciplines_groups` 		ON 	disciplines_groups.DisciplineID = disciplines.ID AND
															disciplines_groups.StudyGroupID = groupID
				LEFT JOIN 	`disciplines_students`		ON 	disciplines_students.DisciplineID = disciplines.ID AND
															disciplines_students.StudentID = StudentID
				WHERE 	disciplines.SemesterID = semID AND
						(	disciplines_students.Type = 'attach'	OR
							(	( disciplines_students.Type IS NULL OR disciplines_students.Type != 'detach' ) AND 
								disciplines_groups.StudyGroupID = groupID 
							)
						)
		)
	ORDER BY modules.OrderNum ASC, submodules.OrderNum ASC;
END //

















-- -------------------------------------------------------------------------------------------
-- ======= Преподаватели =======
-- -------------------------------------------------------------------------------------------


DROP PROCEDURE IF EXISTS GetDisciplineInfoByID//
CREATE PROCEDURE `GetDisciplineInfoByID`(IN `DiscID` INT)
    NO SQL
BEGIN  

	SELECT 	disciplines.AuthorID,
			disciplines.Grade 			AS 'DisciplineGrade',
			disciplines.ExamType,
			disciplines.LectionCount,
			disciplines.PracticeCount,
			disciplines.SemesterID,
			subjects.ID 				AS 'SubjectID',
			subjects.Name 				AS 'SubjectName',
			subjects.Abbr 				AS 'SubjectAbbr'


	FROM `disciplines`
	INNER JOIN `subjects` ON subjects.ID = disciplines.SubjectID
	WHERE disciplines.ID = DiscID;
END //


DROP PROCEDURE IF EXISTS GetTeachersForDiscipline//
CREATE PROCEDURE `GetTeachersForDiscipline`(IN `DiscID` INT)
    NO SQL
BEGIN  

		SELECT 	teachers.ID 			AS 'TeacherID', 
				teachers.LastName 		AS 'TeacherLast', 
				teachers.FirstName 		AS 'TeacherFirst', 
				teachers.SecondName 	AS 'TeacherSecond', 
				degrees.ID 				AS 'DegreeID',
				degrees.Name 			AS 'Degree', 
				departments.ID 			AS 'DepartmentID',
				departments.Name 		AS 'DepartmentName',
				faculties.ID 			AS 'FacultyID', 
				faculties.Name 			AS 'FacultyName', 
				faculties.Abbr 			AS 'FacultyAbbreviation'
		FROM `disciplines_teachers`
		INNER JOIN `teachers` ON teachers.ID = disciplines_teachers.TeacherID
		INNER JOIN `departments` ON departments.ID = teachers.DepartmentID
		INNER JOIN `faculties` ON departments.FacultyID = faculties.ID
		INNER JOIN `degrees` ON degrees.ID = teachers.DegreeID
		WHERE disciplines_teachers.DisciplineID = DiscID;
END //





-- Desc: Вывод всех курсов (ID курса, название курса, базовая студ.группа-слушатель), которые ведет преподаватель
DROP PROCEDURE IF EXISTS GetDisciplinesForTeacher//
CREATE PROCEDURE `GetDisciplinesForTeacher`(IN `TeacherID` INT)
    NO SQL
BEGIN  
    DECLARE semID INT;  						 
	SET semID = GetCurSemesterID();

	SELECT DISTINCT disciplines.ID 			AS 'DisciplineID',
                    disciplines.ExamType 	AS 'DisciplineType',
                    subjects.ID 			AS 'SubjectID',
                    subjects.Name 			AS 'SubjectName',
                    disciplines_teachers.ID AS 'DisciplineTeacherID',
                    study_groups.ID 		AS 'StudyGroupID', 
                    study_groups.GroupNum 	AS 'StudyGroupNum', 
                    study_groups.Grade 		AS 'StudyGroupGrade',
                    study_groups.Name 		AS 'StudyGroupName' 
    FROM `disciplines_groups`
    INNER JOIN `disciplines` 			ON 	disciplines_groups.DisciplineID = disciplines.ID
    INNER JOIN `disciplines_teachers` 	ON 	disciplines.ID = disciplines_teachers.DisciplineID AND 
											disciplines_teachers.TeacherID = TeacherID
    INNER JOIN `subjects`				ON 	subjects.ID = disciplines.SubjectID
    INNER JOIN `study_groups` 			ON 	study_groups.ID = disciplines_groups.StudyGroupID
    WHERE 	disciplines.SemesterID = semID
    ORDER BY 	disciplines.Grade 		ASC, 
    			subjects.Name 			ASC,
    			study_groups.GroupNum 	ASC;
END //






DROP PROCEDURE IF EXISTS GetStudentsForDiscipline//
CREATE PROCEDURE `GetStudentsForDiscipline`	(	IN `TeacherID` INT,
												IN `DisciplineID` INT 
											)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID 
	INTO checker 
	FROM `disciplines_teachers`
	WHERE 	disciplines_teachers.DisciplineID = DisciplineID AND 
			disciplines_teachers.TeacherID = TeacherID
	LIMIT 1;

	IF checker > 0 THEN

		SELECT DISTINCT 	students.ID AS 'StudentID',
							students.LastName,
							students.FirstName,
							students.SecondName,
							study_groups.Grade,
							study_groups.GroupNum,
							(disciplines_students.StudentID IS NOT NULL) AS 'isAttached'
		FROM `students`
		INNER JOIN `study_groups` 			ON students.StudyGroupID = study_groups.ID
		LEFT JOIN 	`disciplines_groups` 		ON 	DisciplineID = disciplines_groups.DisciplineID AND
													study_groups.ID = disciplines_groups.StudyGroupID 
		LEFT JOIN 	`disciplines_students`		ON 	DisciplineID = disciplines_students.DisciplineID AND
													students.ID = disciplines_students.StudentID
		WHERE 	disciplines_students.Type = 'attach'	OR
				(	disciplines_students.Type != 'detach' AND 
					disciplines_groups.StudyGroupID IS NOT NULL 
				);
	ELSE 
		SELECT 	NULL AS 'StudentID',
				NULL AS 'LastName',
				NULL AS 'FirstName',
				NULL AS 'SecondName',
				NULL AS 'Grade',
				NULL AS 'Group'
		LIMIT 0;

	END IF;
END //




-- Вывод всех модулей/подмодулей для данного курса данного преподавателя
DROP PROCEDURE IF EXISTS GetMapForDiscipline//
CREATE PROCEDURE `GetMapForDiscipline`	(	IN `TeacherID` 		INT,
											IN `DisciplineID` 	INT 
										)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID 
	INTO checker 
	FROM `disciplines_teachers`
	WHERE 	disciplines_teachers.DisciplineID = DisciplineID AND 
			disciplines_teachers.TeacherID = TeacherID
	LIMIT 1;

	IF checker > 0 THEN
		SELECT 	subjects.ID 			AS 'SubjectID',
				subjects.Name 			AS 'SubjectName',	
				modules.ID 				AS 'ModuleID', 
				modules.Name 			AS 'ModuleName', 
				submodules.ID 			AS 'SubmoduleID', 
				submodules.Name 		AS 'SubModuleName', 
				submodules.Description 	AS 'SubmoduleDescription', 
				submodules.MaxRate,
				submodules.Type 		AS 'SubmoduleControl'
		FROM `modules`
		LEFT JOIN  `submodules` ON modules.ID = submodules.ModuleID
		INNER JOIN `disciplines` ON modules.DisciplineID = disciplines.ID
		INNER JOIN `subjects` 	ON disciplines.SubjectID = subjects.ID
		ORDER BY modules.OrderNum ASC, 
				 submodules.OrderNum ASC;
 	ELSE 
		SELECT 	NULL AS 'SubjectID',
				NULL AS 'SubjectName',	
				NULL AS 'ModuleID', 
				NULL AS 'ModuleName', 
				NULL AS 'SubmoduleID', 
				NULL AS 'SubModuleName', 
				NULL AS 'SubmoduleDescription', 
				NULL AS 'MaxRate'
		LIMIT 0;
	END IF;
END //





-- Добавление баллов к данному студенту
DROP FUNCTION IF EXISTS SetStudentRate; //
CREATE FUNCTION `SetStudentRate`(	`TeacherID` 	INT,
									`StudentID` 	INT,
									`SubmoduleID` 	INT,
									`Rate` 			INT ) 
									RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker, discID, groupID, semID INT; 
    SET discID = -1;
    SET groupID = -1; 					 
	
    SELECT disciplines.ID 
    INTO discID
	FROM `submodules`
	INNER JOIN `modules` 		ON submodules.ModuleID = modules.ID
	INNER JOIN `disciplines` 	ON disciplines.ID = modules.DisciplineID 
	WHERE submodules.ID = SubmoduleID
	LIMIT 1;

	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON 	disciplines_teachers.DisciplineID = disciplines.ID AND 
											disciplines_teachers.TeacherID = TeacherID  
	WHERE disciplines.ID = discID
	LIMIT 1;

	IF checker > 0 THEN

		SELECT COUNT(rating_table.ID) 
		INTO checker
		FROM `rating_table`
		WHERE 	rating_table.SubmoduleID = SubmoduleID AND 
				rating_table.TeacherID = TeacherID AND 
				rating_table.StudentID = StudentID;

		IF checker = 0 THEN

			SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1; 
			SET semID = GetCurSemesterID(); 
			
			SELECT 	disciplines.ID INTO checker
			FROM `disciplines`
			LEFT JOIN 	`disciplines_groups` 		ON 	disciplines_groups.DisciplineID = disciplines.ID AND
														disciplines_groups.StudyGroupID = groupID
			LEFT JOIN 	`disciplines_students`		ON 	disciplines_students.DisciplineID = disciplines.ID AND
														disciplines_students.StudentID = StudentID
			WHERE 	disciplines.SubjectID = SubjectID 	AND
					disciplines.SemesterID = semID 		AND
					(	disciplines_students.Type = 'attach'	OR
						(disciplines_students.Type != 'detach' AND disciplines_groups.StudyGroupID = groupID )
					)
			LIMIT 1;

			IF checker > 0 THEN
				INSERT INTO `rating_table` ( rating_table.StudentID, rating_table.TeacherID, rating_table.SubmoduleID, rating_table.Rate, rating_table.Date )
				VALUES ( StudentID, TeacherID, SubmoduleID, Rate, CURDATE() );
				RETURN 0;
			END IF;
		END IF;
	END IF;
	RETURN -1;
END //


-- Изменение баллов для данного студента
DROP FUNCTION IF EXISTS ChangeStudentRate//
CREATE FUNCTION `ChangeStudentRate`	(	`TeacherID` 	INT,
										`StudentID` 	INT,
										`SubmoduleID` 	INT,
										`Rate` 			INT 
									) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker, discID, OldVal, accID, groupID, rateID, semID INT;
    SET discID = -1;
    SET groupID = -1;  					 
	
    SELECT disciplines.ID 
    INTO discID
	FROM `submodules`
	INNER JOIN `modules` 	 	ON submodules.ModuleID = modules.ID
	INNER JOIN `disciplines` 	ON disciplines.ID = modules.DisciplineID 
	WHERE submodules.ID = SubmoduleID
	LIMIT 1;

	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON 	disciplines_teachers.DisciplineID = disciplines.ID AND 
											disciplines_teachers.TeacherID = TeacherID  
	WHERE disciplines.ID = discID
	LIMIT 1;

	
	IF checker > 0 THEN
		SELECT rating_table.ID 
		INTO rateID
		FROM `rating_table`
		WHERE 	rating_table.SubmoduleID = SubmoduleID AND 
				rating_table.TeacherID = TeacherID AND 
				rating_table.StudentID = StudentID
		LIMIT 1;

		IF checker > 0 THEN
			SELECT students.StudyGroupID INTO groupID FROM `students` WHERE students.ID = StudentID LIMIT 1; 
			SET semID = GetCurSemesterID(); 
			
			SELECT 	disciplines.ID INTO checker
			FROM `disciplines`
			LEFT JOIN 	`disciplines_groups` 		ON 	disciplines_groups.DisciplineID = disciplines.ID AND
														disciplines_groups.StudyGroupID = groupID
			LEFT JOIN 	`disciplines_students`		ON 	disciplines_students.DisciplineID = disciplines.ID AND
														disciplines_students.StudentID = StudentID
			WHERE 	disciplines.SubjectID = SubjectID 	AND
					disciplines.SemesterID = semID 		AND
					(	disciplines_students.Type = 'attach'	OR
						(disciplines_students.Type != 'detach' AND disciplines_groups.StudyGroupID = groupID )
					)
			LIMIT 1;

			IF checker > 0 THEN	
				SELECT rating_table.Rate 
				INTO OldVal
				FROM `rating_table`
				WHERE rating_table.ID = rateID
				LIMIT 1;


				SELECT accounts.ID 
				INTO accID
				FROM `teachers`
				INNER JOIN `accounts` ON teachers.AccountID = accounts.ID
				WHERE teachers.ID = TeacherID
				LIMIT 1; 

				INSERT INTO `logs` ( logs.RatingTableID, logs.AccountID, logs.OldValue, logs.NewValue, logs.Date )
				VALUES ( rateID, accID, OldVal, Rate, NOW() );
			

				UPDATE `rating_table`
				SET rating_table.Rate = Rate
				WHERE rating_table.ID = rateID;

				RETURN 0;


			END IF;
		END IF;
	END IF;
	RETURN -1;
END //













-- -------------------------------------------------------------------------------------------
-- ====== disciplines ============= --
-- -------------------------------------------------------------------------------------------


DROP FUNCTION IF EXISTS BindTeacher//
CREATE FUNCTION `BindTeacher`(	`AccessedTeacher` INT,
								`BindingTeacher` INT,
								`DisciplineID` INT		
							 ) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT; 

	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON 	disciplines_teachers.DisciplineID = disciplines.ID AND 
											disciplines_teachers.TeacherID = AccessedTeacher  
	WHERE disciplines.ID = DisciplineID
	LIMIT 1;
	IF NOT (checker > 0) THEN
		RETURN -1;
	END IF;



	SET checker = -1;
	SELECT disciplines_teachers.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID = BindingTeacher  
	WHERE disciplines.ID = DisciplineID
	LIMIT 1;
	IF checker > 0 THEN
		RETURN 0;
	END IF;

	INSERT INTO `disciplines_teachers`
		(	disciplines_teachers.DisciplineID,
			disciplines_teachers.TeacherID 	)
	VALUES	( DisciplineID, BindingTeacher );
	RETURN 0;
	
END //


DROP FUNCTION IF EXISTS UnbindTeacher//
CREATE FUNCTION `UnbindTeacher`	(	`AccessedTeacher` INT,
									`BindingTeacher` INT,
									`DisciplineID` INT		
							 	) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT; 

	SET checker = -1;
	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	WHERE 	disciplines.ID = DisciplineID AND
			disciplines.AuthorID = AccessedTeacher 
	LIMIT 1;
	IF NOT (checker > 0) THEN
		RETURN -1;
	END IF;

	DELETE FROM `disciplines_teachers`
	WHERE 	disciplines_teachers.DisciplineID = DisciplineID AND
			disciplines_teachers.TeacherID = BindingTeacher;
	RETURN 0;
	
END //


DROP FUNCTION IF EXISTS AddDiscipline//
CREATE FUNCTION `AddDiscipline`	(	`TeacherID` 	INT,
									`Grade`			INT,
									`SubjectID`		INT,
									`ExamType`		VARCHAR(30),
									`LectionCount`  INT,
									`PracticeCount` INT,
									`DepartmentID`	INT		
							 	)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE semID, checker INT; 
    SET semID = GetCurSemesterID();   

	IF NOT (Grade > 0) THEN
		RETURN -1;
	END IF; 

    SELECT departments.ID
    INTO checker
    FROM `departments`
    WHERE departments.ID = DepartmentID
    LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;
	

    SELECT subjects.ID
    INTO checker
    FROM `subjects`
    WHERE subjects.ID = SubjectID
    LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;



	INSERT INTO `disciplines` 	(	disciplines.AuthorID, 
									disciplines.Grade, 
									disciplines.SubjectID, 
									disciplines.ExamType, 
									disciplines.LectionCount, 
									disciplines.PracticeCount,
									disciplines.SemesterID,
									disciplines.DepartmentID )
	VALUES 					( TeacherID, Grade, SubjectID, ExamType, LectionCount, PracticeCount, semID, DepartmentID );
	
	-- RETURN 	(	SELECT disciplines.ID
	-- 			FROM   `disciplines`
	-- 			WHERE 	disciplines.AuthorID  	= TeacherID 	AND
	-- 					disciplines.SubjectID 	= SubjectID		AND
	-- 					disciplines.Grade 		= Grade
	-- 			ORDER BY disciplines.ID DESC
	-- 			LIMIT 1
	-- 		);
	SET checker = LAST_INSERT_ID();


	INSERT INTO `disciplines_teachers`
		(	disciplines_teachers.DisciplineID,
			disciplines_teachers.TeacherID 	)
	VALUES	( checker, TeacherID );

	RETURN checker;
	
	
END //



DROP FUNCTION IF EXISTS ChangeDiscipline//
CREATE FUNCTION `ChangeDiscipline`	(	`TeacherID` 	INT,
										`DisciplineID`	INT,
										`ExamType`		VARCHAR(30),
										`LectionCount`  INT,
										`PracticeCount` INT,
										`Grade`			INT,
										`SubjectID`		INT		
							 		) 	RETURNS int(11)
    NO SQL
BEGIN 
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID
	INTO checker 
	FROM `disciplines_teachers`
	WHERE disciplines_teachers.DisciplineID = DisciplineID AND disciplines_teachers.TeacherID 	= TeacherID
	LIMIT 1;
	IF NOT (checker > 0) THEN 
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT rating_table.ID
	INTO checker
	FROM `rating_table`
	INNER JOIN `submodules` ON submodules.ID = rating_table.SubmoduleID
	INNER JOIN `modules` ON modules.ID = submodules.ModuleID
	WHERE modules.DisciplineID = DisciplineID
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF;


	UPDATE `disciplines`
	SET 	disciplines.ExamType = ExamType,
			disciplines.LectionCount = LectionCount,
			disciplines.PracticeCount = PracticeCount,
			disciplines.Grade = Grade,
			disciplines.SubjectID = SubjectID
	WHERE disciplines.ID = DisciplineID;

	RETURN 0;

END //


DROP FUNCTION IF EXISTS BindGroup//
CREATE FUNCTION `BindGroup`	(	`TeacherID` 	INT,
								`DisciplineID` 	INt,
								`StudyGroupID` 	INT
					 		) 	RETURNS int(11)
    NO SQL
BEGIN 
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID
	INTO checker 
	FROM `disciplines_teachers`
	WHERE disciplines_teachers.DisciplineID = DisciplineID AND disciplines_teachers.TeacherID 	= TeacherID
	LIMIT 1;
	IF NOT (checker > 0) THEN 
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT study_groups.ID
	INTO checker 
	FROM `study_groups`
	WHERE study_groups.ID = StudyGroupID
	LIMIT 1;
	IF NOT (checker > 0) THEN 
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT disciplines_groups.ID
	INTO checker
	FROM `disciplines_groups`
	WHERE 	disciplines_groups.StudyGroupID = StudyGroupID AND
			disciplines_groups.DisciplineID = DisciplineID
	LIMIT 1;
	IF checker > 0 THEN
		RETURN 0;
	END IF;

	INSERT INTO `disciplines_groups` 	(	disciplines_groups.DisciplineID,
											disciplines_groups.StudyGroupID
										)
	VALUES 								( DisciplineID, StudyGroupID );

	RETURN 0;

END //



DROP FUNCTION IF EXISTS BindStudent//
CREATE FUNCTION `BindStudent`	(	`TeacherID` 	INT,
									`DisciplineID` 	INt,
									`StudentID` 	INT
					 			) 	RETURNS int(11)
    NO SQL
BEGIN 
    DECLARE checker, SG, temp INT;  					 
	
    SELECT students.StudyGroupID
    INTO SG
    FROM `students`
    WHERE students.ID = StudentID
    LIMIT 1;
    

	SELECT disciplines_teachers.ID
	INTO checker 
	FROM `disciplines_teachers`
	WHERE disciplines_teachers.DisciplineID = DisciplineID AND disciplines_teachers.TeacherID 	= TeacherID
	LIMIT 1;
	IF NOT (checker > 0) THEN 
		RETURN -1;
	END IF;


	SELECT disciplines_groups.ID
	INTO temp
	FROM `disciplines_groups`
	WHERE 	disciplines_groups.DisciplineID = DisciplineID AND
			disciplines_groups.StudyGroupID = SG
	LIMIT 1;


	IF temp > 0 THEN
		DELETE FROM `disciplines_students`
		WHERE 	disciplines_students.DisciplineID = DisciplineID AND
				disciplines_students.StudentID = StudentID;
	ELSE 
		INSERT INTO `disciplines_students`	(	disciplines_students.DisciplineID,
												disciplines_students.StudentID,
												disciplines_students.Type
											)
		VALUES								( DisciplineID, StudentID, 'attach');

	END IF;
	RETURN 0;

END //



DROP FUNCTION IF EXISTS UnbindGroup//
CREATE FUNCTION `UnbindGroup`	(	`TeacherID` 	INT,
									`DisciplineID` 	INt,
									`StudyGroupID` 	INT
					 			) 	RETURNS int(11)
    NO SQL
BEGIN 
    DECLARE checker INT;  					 
	
	SELECT disciplines_teachers.ID
	INTO checker 
	FROM `disciplines_teachers`
	WHERE disciplines_teachers.DisciplineID = DisciplineID AND disciplines_teachers.TeacherID 	= TeacherID
	LIMIT 1;
	IF NOT (checker > 0) THEN 
		RETURN -1;
	END IF;

	SET checker = -1;
	SELECT study_groups.ID
	INTO checker 
	FROM `study_groups`
	WHERE study_groups.ID = StudyGroupID
	LIMIT 1;
	IF NOT (checker > 0) THEN 
		RETURN -1;
	END IF;

	DELETE FROM `disciplines_groups`
	WHERE 	disciplines_groups.DisciplineID = DisciplineID AND
			disciplines_groups.StudyGroupID = StudyGroupID;

	RETURN 0;

END //



DROP FUNCTION IF EXISTS UnbindStudent//
CREATE FUNCTION `UnbindStudent`	(	`TeacherID` 	INT,
									`DisciplineID` 	INt,
									`StudentID` 	INT
					 			) 	RETURNS int(11)
    NO SQL
BEGIN 
    DECLARE checker, SG, temp INT;  					 
	
    SELECT students.StudyGroupID
    INTO SG
    FROM `students`
    WHERE students.ID = StudentID
    LIMIT 1;
    

	SELECT disciplines_teachers.ID
	INTO checker 
	FROM `disciplines_teachers`
	WHERE disciplines_teachers.DisciplineID = DisciplineID AND disciplines_teachers.TeacherID 	= TeacherID
	LIMIT 1;
	IF NOT (checker > 0) THEN 
		RETURN -1;
	END IF;


	SELECT disciplines_groups.ID
	INTO temp
	FROM `disciplines_groups`
	WHERE 	disciplines_groups.DisciplineID = DisciplineID AND
			disciplines_groups.StudyGroupID = SG
	LIMIT 1;


	IF temp > 0 THEN
		INSERT INTO `disciplines_students`	(	disciplines_students.DisciplineID,
												disciplines_students.StudentID,
												disciplines_students.Type
											)
		VALUES								( DisciplineID, StudentID, 'detach');

	ELSE 
		DELETE FROM `disciplines_students`
		WHERE 	disciplines_students.DisciplineID = DisciplineID AND
				disciplines_students.StudentID = StudentID;

	END IF;
	RETURN 0;

END //














-- -------------------------------------------------------------------------------------------
-- ====== Modules ============= --
-- -------------------------------------------------------------------------------------------




-- Внесение изменений в модуль (Title, Order и т.д.), который неявно прикреплен к преподавателю
DROP FUNCTION IF EXISTS ChangeModule//
CREATE FUNCTION `ChangeModule`	(	`TeacherID` INT, 
									`ModuleID` INT, 
									`OrderNum` INT, 
									`Name` VARCHAR(50) 
								) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	SET checker = -1;
	SELECT modules.ID 
	INTO checker
	FROM `modules`
	WHERE modules.DisciplineID = checker AND modules.OrderNum = OrderNum;
	IF checker > 0 THEN
		RETURN -1;
	END IF;

	UPDATE `modules` 
	SET modules.OrderNum 	= OrderNum, 
		modules.Name 		= Name
	WHERE modules.ID = ModuleID;
	RETURN 0;
END //


DROP FUNCTION IF EXISTS ChangeModuleName//
CREATE FUNCTION `ChangeModuleName`	(	`TeacherID` INT, 
										`ModuleID` INT, 
										`Name` VARCHAR(50) 
									) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	UPDATE `modules` 
	SET modules.Name 		= Name
	WHERE modules.ID = ModuleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeModuleOrder//
CREATE FUNCTION `ChangeModuleOrder`	(	`TeacherID` INT, 
										`ModuleID` INT, 
										`OrderNum` INT 
									) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	SET checker = -1;
	SELECT modules.ID 
	INTO checker
	FROM `modules`
	WHERE modules.DisciplineID = checker AND modules.OrderNum = OrderNum;
	IF checker > 0 THEN
		RETURN -1;
	END IF;

	UPDATE `modules` 
	SET modules.OrderNum 	= OrderNum
	WHERE modules.ID = ModuleID;
	RETURN 0;
END //

-- Добавление нового модуля
DROP FUNCTION IF EXISTS AddModule//
CREATE FUNCTION `AddModule`	(	`TeacherID` 	INT, 
								`DisciplineID` 	INT, 
								`OrderNum` 		INT, 
								`Name` 			VARCHAR(50) 
							) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT disciplines.ID 
	INTO checker 
	FROM `disciplines`
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.TeacherID = TeacherID AND disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	disciplines.ID = DisciplineID
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	SET checker = -1;
	SELECT modules.ID 
	INTO checker
	FROM `modules`
	WHERE modules.DisciplineID = DisciplineID AND modules.OrderNum = OrderNum;
	IF checker > 0 THEN
		RETURN -2;
	END IF;


	INSERT INTO `modules` 	( modules.Name, modules.OrderNum, modules.DisciplineID )
	VALUES 					( Name , OrderNum , DisciplineID );
	

	RETURN 	(	SELECT 	modules.ID
				FROM `modules`
				WHERE 	modules.DisciplineID = DisciplineID AND 
						modules.OrderNum = OrderNum
			);


END //

-- Добавление нового подмодуля к данному модулю
DROP FUNCTION IF EXISTS DeleteModule//
CREATE FUNCTION `DeleteModule`	(	`TeacherID` INT,
									`ModuleID` 	INT 
								)  	RETURNS int(11)
    NO SQL
BEGIN  

    DECLARE checker INT;

	SELECT disciplines.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON 	disciplines_teachers.TeacherID = TeacherID AND 
											disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	modules.ID = ModuleID
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;


	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = ModuleID
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF;


	DELETE FROM `modules` 
	WHERE modules.ID = ModuleID; 


	RETURN 0;
END //






DROP FUNCTION IF EXISTS SwapModuleOrder//
CREATE FUNCTION `SwapModuleOrder`	(	`TeacherID` INT, 
										`ModuleID1` INT, 
										`ModuleID2` INT 
									) 	RETURNS int(11)
    NO SQL
BEGIN  
   	DECLARE checker, ord1, ord2, disc1, disc2 INT; 				 
	
	SELECT COUNT(modules.ID) 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID 
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID  AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE modules.ID = ModuleID1 OR modules.ID = ModuleID2
	LIMIT 2;
	IF checker != 2 THEN
		RETURN -1;
	END IF;

	SELECT 	modules.OrderNum,
			modules.DisciplineID
	INTO ord1, disc1
	FROM `modules`
	WHERE modules.ID = ModuleID1
	LIMIT 1;

	SELECT 	modules.OrderNum,
			modules.DisciplineID
	INTO ord2, disc2
	FROM `modules`
	WHERE modules.ID = ModuleID2
	LIMIT 1;

	IF disc1 != disc2 THEN
		RETURN -1;
	END IF;

	UPDATE `modules` 
	SET modules.OrderNum 	= 13666
	WHERE modules.ID = ModuleID1;

	UPDATE `modules` 
	SET modules.OrderNum 	= ord1
	WHERE modules.ID = ModuleID2;

	UPDATE `modules` 
	SET modules.OrderNum 	= ord2
	WHERE modules.ID = ModuleID1;
	
	RETURN 0;
END //






















-- -------------------------------------------------------------------------------------------
-- ====== Submodules ============= --
-- -------------------------------------------------------------------------------------------


-- Внесение изменений в подмодуль (MaxRate, TItle, Description, Order и т.д.), который неявно прикреплен к преподавателю
DROP FUNCTION IF EXISTS ChangeSubmodule//
CREATE FUNCTION `ChangeSubmodule`	(	`TeacherID` 	INT, 
										`SubmoduleID` 	INT, 
										`MaxRate` 		INT, 
										`OrderNum` 		INT, 
										`Name` 			VARCHAR(50), 
										`Description` 	VARCHAR(200),
										`ControlType`	VARCHAR(30) 
									) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = checker AND submodules.OrderNum = OrderNum
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF; 

	UPDATE `submodules` 
	SET submodules.MaxRate 		= MaxRate, 
		submodules.OrderNum 	= OrderNum, 
		submodules.Name 		= Name, 
		submodules.Description 	= Description,
		submodules.Type 		= ControlType
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleControlType//
CREATE FUNCTION `ChangeSubmoduleControlType`(	`TeacherID` 	INT, 
												`SubmoduleID` 	INT, 
												`ControlType`	VARCHAR(30) 
											)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET submodules.Type 		= ControlType
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleMaxRate//
CREATE FUNCTION `ChangeSubmoduleMaxRate`	(	`TeacherID` 	INT, 
												`SubmoduleID` 	INT, 
												`MaxRate` 		INT
											) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET submodules.MaxRate 		= MaxRate
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleOrder//
CREATE FUNCTION `ChangeSubmoduleOrder`	(	`TeacherID` 	INT, 
											`SubmoduleID` 	INT, 
											`OrderNum` 		INT
										)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;


	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = checker AND submodules.OrderNum = OrderNum
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF; 


	UPDATE `submodules` 
	SET submodules.OrderNum 	= OrderNum
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleName//
CREATE FUNCTION `ChangeSubmoduleName`	(	`TeacherID` 	INT, 
											`SubmoduleID` 	INT, 
											`Name` 			VARCHAR(50) 
										)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET 	submodules.Name 		= Name
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //

DROP FUNCTION IF EXISTS ChangeSubmoduleDescription//
CREATE FUNCTION `ChangeSubmoduleDescription`	(	`TeacherID` 	INT, 
													`SubmoduleID` 	INT, 
													`Description` 	VARCHAR(200) 
												) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT submodules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID 
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET submodules.Description 	= Description
	WHERE submodules.ID = SubmoduleID;
	RETURN 0;
END //



DROP FUNCTION IF EXISTS DeleteSubmodule//
CREATE FUNCTION `DeleteSubmodule`	(	`TeacherID` 	INT,
										`SubmoduleID` 	INT
									)  	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;

	SELECT modules.ID 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` ON modules.ID = submodules.ModuleID
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.TeacherID = TeacherID AND disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	submodules.ID = SubmoduleID
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;



	SET checker = -1;
	SELECT rating_table.ID
	INTO checker
	FROM `rating_table`
	WHERE rating_table.SubmoduleID = SubmoduleID
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF;



	DELETE FROM `submodules` 
	WHERE submodules.ID = SubmoduleID; 

	RETURN 0;
END //


DROP FUNCTION IF EXISTS AddSubmodule//
CREATE FUNCTION `AddSubmodule`	(	`TeacherID` 	INT, 
									`ModuleID` 		INT, 
									`MaxRate` 		INT, 
									`OrderNum` 		INT, 
									`Name` 			VARCHAR(50), 
									`Description`	VARCHAR(200),
									`ControlType`	VARCHAR(30)
								) 	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker INT;  					 
	
	SELECT modules.ID 
	INTO checker 
	FROM `modules`
	INNER JOIN `disciplines` ON disciplines.ID = modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.TeacherID = TeacherID AND disciplines_teachers.DisciplineID = disciplines.ID
	WHERE 	modules.ID = ModuleID
	LIMIT 1;
    IF NOT (checker > 0) THEN
    	RETURN -1;
	END IF;


	SET checker = -1;
	SELECT submodules.ID
	INTO checker
	FROM `submodules`
	WHERE submodules.ModuleID = ModuleID AND submodules.OrderNum = OrderNum
	LIMIT 1;
	IF checker > 0 THEN
		RETURN -1;
	END IF; 

	INSERT INTO `submodules` 	( submodules.ModuleID, submodules.MaxRate, submodules.OrderNum, submodules.Name, submodules.Description, submodules.Type )
	VALUES						( ModuleID, MaxRate, OrderNum, Name, Description, ControlType);
	
	RETURN 	(	SELECT submodules.ID
				FROM `submodules`
				WHERE 	submodules.ModuleID = ModuleID AND
						submodules.OrderNum = OrderNum
			);
END //


DROP FUNCTION IF EXISTS SwapSubmoduleOrder//
CREATE FUNCTION `SwapSubmoduleOrder`(	`TeacherID` 	INT, 
										`SubmoduleID1` 	INT, 
										`SubmoduleID2` 	INT
									)	RETURNS int(11)
    NO SQL
BEGIN  
    DECLARE checker, ord1, ord2, mod1, mod2 INT;  					 
	
	SELECT COUNT(submodules.ID) 
	INTO checker 
	FROM `submodules`
	INNER JOIN `modules` 		ON 	submodules.ModuleID 	= modules.ID
	INNER JOIN `disciplines` 	ON 	disciplines.ID 			= modules.DisciplineID
	INNER JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = disciplines.ID AND disciplines_teachers.TeacherID 	= TeacherID
	WHERE 	submodules.ID = SubmoduleID1 OR submodules.ID = SubmoduleID2;
	IF checker != 2 THEN
		RETURN -1;
	END IF;


	SELECT 	submodules.OrderNum,
			submodules.ModuleID
	INTO ord1, mod1
	FROM `submodules`
	WHERE submodules.ID = SubmoduleID1
	LIMIT 1;

	SELECT 	submodules.OrderNum,
			submodules.ModuleID
	INTO ord2, mod2
	FROM `submodules`
	WHERE submodules.ID = SubmoduleID2
	LIMIT 1;

	IF mod1 != mod2 THEN
		RETURN -1;
	END IF;

	UPDATE `submodules` 
	SET submodules.OrderNum 	= 13666
	WHERE submodules.ID = SubmoduleID1;

	UPDATE `submodules` 
	SET submodules.OrderNum 	= ord1
	WHERE submodules.ID = SubmoduleID2;

	UPDATE `submodules` 
	SET submodules.OrderNum 	= ord2
	WHERE submodules.ID = SubmoduleID1;
	RETURN 0;
END //










-- -------------------------------------------------------------------------------------------
-- ====== Requests =======  --
-- -------------------------------------------------------------------------------------------

-- DROP PROCEDURE IF EXISTS GetLastRequests//
-- CREATE PROCEDURE `GetLastRequests`	( 	IN `CountN` INT 
-- 									)
--     NO SQL
-- BEGIN 
-- 	DECLARE offsetN, allNum, countClone INT;
-- 	SELECT COUNT(requests.ID) INTO allNum FROM `requests`;
-- 	SET countClone = CountN;
-- 	IF CountN = -1 THEN 
-- 		SET countClone = allNum;
-- 	END IF;

-- 	SET offsetN = allNum - CountN;
-- 	IF offsetN < 0 THEN
-- 		SET offsetN = 0;
-- 	END IF;

-- 	SELECT 	requests.ID,
-- 			requests.AccountID,
-- 			requests.Information,
-- 			requests.Date,
-- 			requests.Status
-- 	FROM `requests`
-- 	WHERE requests.Status = 0
-- 	LIMIT countClone OFFSET offsetN;

-- END //


-- DROP PROCEDURE IF EXISTS GetFirstRequests//
-- CREATE PROCEDURE `GetFirstRequests`	( 	IN `CountN` INT 
-- 									)
--     NO SQL
-- BEGIN 
-- 	DECLARE allNum, countClone INT;
-- 	SELECT COUNT(requests.ID) INTO allNum FROM `requests`;
-- 	SET countClone = CountN;
-- 	IF CountN = -1 THEN 
-- 		SET countClone = allNum;
-- 	END IF;

-- 	SELECT 	requests.ID,
-- 			requests.AccountID,
-- 			requests.Information,
-- 			requests.Date,
-- 			requests.Status
-- 	FROM `requests`
-- 	WHERE requests.Status = 0
-- 	LIMIT countClone;
-- END //




DELIMITER ;